<?php

namespace Pogodi\Product;

use DateTime;
use InvalidArgumentException;
use Nette\Database\Context;
use Nette\Object;

class Product extends Object
{

    const
        TABLE_NAME = 'product';

    /**
     * @var Context
     */
    protected $database;

    /**
     * @var int
     */
    protected $id;

    /**
     * @var string
     */
    protected $title;

    /**
     * @var string
     */
    protected $sku;

    /**
     * @var int
     */
    protected $price;

    /**
     * @var Category
     */
    protected $category;

    /**
     * @var Brand
     */
    protected $brand;

    /**
     * @var boolean
     */
    protected $active;

    /**
     * @var DateTime
     */
    protected $created;

    /**
     * @var DateTime
     */
    protected $updated;

    /**
     * @var Warehouse
     */
    protected $warehouses;

    public function __construct(Context $context)
    {
        $this->database = $context;
        $this->id = NULL;
        $this->active = FALSE;

        $this->warehouses = new Warehouse($context);
        $this->category = new Category($context);
        $this->brand = new Brand($context);
    }

    public function loadById($id)
    {
        $row = $this->database->table(self::TABLE_NAME)->where('id', $id)->fetch();
        if (!$row) {
            throw new InvalidArgumentException("Product with ID '$id' not exist.");
        }
        $this->id = intval($row['id']);
        $this->title = $row['title'];
        $this->sku = $row['sku'];
        $this->price = $row['price'];
        $this->active = $row['active'] ? TRUE : FALSE;
        $this->created = $row['created'];
        $this->updated = $row['updated'];

        $this->warehouses->loadById($id);

        $this->category->loadByCategoryId($row['category']);
        if (NULL !== $row['brand']) {
            $this->brand->loadByBrandId($row['brand']);
        }
    }

    public function save()
    {
        $new_product = $this->id ? FALSE : TRUE;
        $data = [
            'sku' => $this->sku,
            'price' => $this->price,
            'category' => $this->getCategory()->getId(),
            'brand' => $this->getBrand()->getId(),
            'active' => $this->active,
            'title' => $this->title,
            'updated' => new DateTime(),
        ];
        $this->persist($data, $new_product);

        $this->getWarehouses()->setProductId($this->id);
        $this->getWarehouses()->save();
    }

    public function delete()
    {
        if (!$this->id) {
            throw new InvalidArgumentException("No product ID set to delete");
        }
        $this->database->table(self::TABLE_NAME)->where('id', $this->id)->delete();
    }

    protected function persist($data, $new_product)
    {
        $this->updated = new DateTime();
        if ($new_product) {
            $this->created = $data['updated'];;
            $data['created'] = $data['updated'];
            $row = $this->database->table(self::TABLE_NAME)->insert($data);
            $this->id = $row['id'];
        } else {
            $this->database->table(self::TABLE_NAME)->where('id', $this->id)->update($data);
        }
    }

    /**
     * @param int $category
     */
    public function setCategoryId($category)
    {
        $this->category->loadByCategoryId($category);
    }

    /**
     * @param int $brand_id
     */
    public function setBrandId($brand_id)
    {
        if (NULL === $brand_id) {
            $this->brand->setNoBrand();
        } else {
            $this->brand->loadByBrandId($brand_id);
        }
    }

    /**
     * @return Brand
     */
    public function getBrand()
    {
        return $this->brand;
    }


    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }


    /**
     * @return string
     */
    public function getSku()
    {
        return $this->sku;
    }

    /**
     * @param string $sku
     */
    public function setSku($sku)
    {
        $this->sku = $sku;
    }

    /**
     * @return boolean
     */
    public function isActive()
    {
        return $this->active;
    }

    /**
     * @param boolean $active
     */
    public function setActive($active)
    {
        $this->active = $active;
    }

    /**
     * @return DateTime
     */
    public function getCreated()
    {
        return $this->created;
    }

    /**
     * @return DateTime
     */
    public function getUpdated()
    {
        return $this->updated;
    }

    /**
     * @return Warehouse
     */
    public function getWarehouses()
    {
        return $this->warehouses;
    }

    /**
     * @return Category
     */
    public function getCategory()
    {
        return $this->category;
    }

    /**
     * @return int
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param int $price
     */
    public function setPrice($price)
    {
        $this->price = (int)$price;
    }


}